@@ -34,10 +34,10 @@ import re
 import shutil
 from abc import ABCMeta, abstractmethod
 from contextlib import closing
+from distutils.version import LooseVersion as Version
 from functools import partial
 
 import dateutil.parser
-from distutils.version import LooseVersion as Version
 
 from barman import output, xlog
 from barman.cloud_providers import get_snapshot_interface_from_server_config
@@ -49,18 +49,24 @@ from barman.exceptions import (
     BackupException,
     CommandFailedException,
     DataTransferFailure,
+    FileNotFoundException,
     FsOperationFailed,
     PostgresConnectionError,
+    PostgresConnectionLost,
     PostgresIsInRecovery,
     SnapshotBackupException,
     SshCommandException,
-    FileNotFoundException,
 )
 from barman.fs import UnixLocalCommand, UnixRemoteCommand, unix_command_factory
 from barman.infofile import BackupInfo
+from barman.postgres import PostgresKeepAlive
 from barman.postgres_plumbing import EXCLUDE_LIST, PGDATA_EXCLUDE_LIST
 from barman.remote_status import RemoteStatusMixin
 from barman.utils import (
+    check_aws_expiration_date_format,
+    check_aws_snapshot_lock_cool_off_period_range,
+    check_aws_snapshot_lock_duration_range,
+    check_aws_snapshot_lock_mode,
     force_str,
     human_readable_timedelta,
     mkpath,
@@ -574,6 +580,7 @@ class PostgresBackupExecutor(BackupExecutor):
 
         :param barman.infofile.LocalBackupInfo backup_info: backup information
         """
+
         # Make sure the destination directory exists, ensure the
         # right permissions to the destination dir
         backup_dest = backup_info.get_data_directory()
@@ -608,6 +615,13 @@ class PostgresBackupExecutor(BackupExecutor):
         # for the whole duration of the copy
         self.server.close()
 
+        # Find the backup_manifest file path of the parent backup in case
+        # it is an incremental backup
+        parent_backup_info = backup_info.get_parent_backup_info()
+        parent_backup_manifest_path = None
+        if parent_backup_info:
+            parent_backup_manifest_path = parent_backup_info.get_backup_manifest_path()
+
         pg_basebackup = PgBaseBackup(
             connection=self.server.streaming,
             destination=backup_dest,
@@ -624,6 +638,7 @@ class PostgresBackupExecutor(BackupExecutor):
             compression=self.backup_compression,
             err_handler=self._err_handler,
             out_handler=PgBaseBackup.make_logging_handler(logging.INFO),
+            parent_backup_manifest_path=parent_backup_manifest_path,
         )
 
         # Do the actual copy
@@ -857,6 +872,7 @@ class ExternalBackupExecutor(with_metaclass(ABCMeta, BackupExecutor)):
             self._update_action_from_strategy()
             raise
 
+        connection_error = False
         try:
             # save any metadata changed by start_backup() call
             # This must be inside the try-except, because it could fail
@@ -885,20 +901,34 @@ class ExternalBackupExecutor(with_metaclass(ABCMeta, BackupExecutor)):
             # the begin_wal value is surely known. Doing it twice is safe
             # because this function is useful only during the first backup.
             self._purge_unused_wal_files(backup_info)
-        except BaseException:
+
+        except PostgresConnectionLost:
+            # This exception is most likely to be raised by the PostgresKeepAlive,
+            # meaning that we lost the connection (and session) during the backup.
+            connection_error = True
+            raise
+        except BaseException as ex:
             # we do not need to do anything here besides re-raising the
             # exception. It will be handled in the external try block.
             output.error("The backup has failed %s", self.current_action)
+            # As we have found that in certain corner cases the exception
+            # passing through this block is not logged or even hidden by
+            # other exceptions happening in the finally block, we are adding a
+            # debug log line to make sure that the exception is visible.
+            _logger.debug("Backup failed: %s" % ex, exc_info=True)
             raise
         else:
             self.current_action = "issuing stop of the backup"
         finally:
-            output.info("Asking PostgreSQL server to finalize the backup.")
-            try:
-                self.strategy.stop_backup(backup_info)
-            except BaseException:
-                self._update_action_from_strategy()
-                raise
+            # If a connection error has been raised, it means we lost our session in the
+            # Postgres server. In such cases, it's useless to try a backup stop command.
+            if not connection_error:
+                output.info("Asking PostgreSQL server to finalize the backup.")
+                try:
+                    self.strategy.stop_backup(backup_info)
+                except BaseException:
+                    self._update_action_from_strategy()
+                    raise
 
     def _local_check(self, check_strategy):
         """
@@ -1182,6 +1212,26 @@ class RsyncBackupExecutor(ExternalBackupExecutor):
                 "backup_compression option is not supported by rsync backup_method"
             )
 
+    def backup(self, *args, **kwargs):
+        """
+        Perform an Rsync backup.
+
+        .. note::
+            This method currently only calls the parent backup method but inside a keepalive
+            context to ensure the connection does not become idle long enough to get dropped
+            by a firewall, for instance. This is important to ensure that ``pg_backup_start()``
+            and ``pg_backup_stop()`` are called within the same session.
+        """
+        try:
+            with PostgresKeepAlive(
+                self.server.postgres, self.config.keepalive_interval, True
+            ):
+                super(RsyncBackupExecutor, self).backup(*args, **kwargs)
+        except PostgresConnectionLost:
+            raise BackupException(
+                "Connection to the Postgres server was lost during the backup."
+            )
+
     def backup_copy(self, backup_info):
         """
         Perform the actual copy of the backup using Rsync.
@@ -1489,6 +1539,59 @@ class SnapshotBackupExecutor(ExternalBackupExecutor):
                     "%s option is required by snapshot backup_method" % config_var
                 )
 
+        # Check if aws_snapshot_lock_mode is set with snapshot_provider = aws.
+        if (
+            getattr(self.server.config, "aws_snapshot_lock_mode")
+            and getattr(self.server.config, "snapshot_provider") == "aws"
+        ):
+            self._validate_aws_lock_configuration()
+
+    def _validate_aws_lock_configuration(self):
+        """Verify configuration is valid for locking a snapshot backup using AWS
+        provider.
+        """
+        if not (
+            getattr(self.server.config, "aws_snapshot_lock_duration")
+            or getattr(self.server.config, "aws_snapshot_lock_expiration_date")
+        ):
+            self.server.config.update_msg_list_and_disable_server(
+                "'aws_snapshot_lock_mode' is set, you must specify "
+                "'aws_snapshot_lock_duration' or 'aws_snapshot_lock_expiration_date'."
+            )
+
+        if getattr(self.server.config, "aws_snapshot_lock_duration") and getattr(
+            self.server.config, "aws_snapshot_lock_expiration_date"
+        ):
+            self.server.config.update_msg_list_and_disable_server(
+                "You must specify either 'aws_snapshot_lock_duration' or "
+                "'aws_snapshot_lock_expiration_date' in the configuration, but not both."
+            )
+
+        if getattr(
+            self.server.config, "aws_snapshot_lock_mode"
+        ) == "governance" and getattr(
+            self.server.config, "aws_snapshot_lock_cool_off_period"
+        ):
+            self.server.config.update_msg_list_and_disable_server(
+                "'aws_snapshot_lock_cool_off_period' cannot be used with "
+                "'aws_snapshot_lock_mode' = 'governance'."
+            )
+
+        lock_args = {
+            "aws_snapshot_lock_cool_off_period": check_aws_snapshot_lock_cool_off_period_range,
+            "aws_snapshot_lock_duration": check_aws_snapshot_lock_duration_range,
+            "aws_snapshot_lock_mode": check_aws_snapshot_lock_mode,
+            "aws_snapshot_lock_expiration_date": check_aws_expiration_date_format,
+        }
+        for arg, parser in lock_args.items():
+            lock_arg = getattr(self.server.config, arg)
+            if lock_arg:
+                try:
+                    _ = parser(lock_arg)
+                except Exception as e:
+                    error_message = str(e)
+                    self.server.config.update_msg_list_and_disable_server(error_message)
+
     @staticmethod
     def add_mount_data_to_volume_metadata(volumes, remote_cmd):
         """
@@ -1771,6 +1874,20 @@ class BackupStrategy(with_metaclass(ABCMeta, object)):
                 msg = "\t%s, %s, %s" % (item.oid, item.name, item.location)
                 _logger.info(msg)
 
+        # Set data_checksums state
+        data_checksums = self.postgres.get_setting("data_checksums")
+        backup_info.set_attribute("data_checksums", data_checksums)
+
+        # Get summarize_wal information for incremental backups
+        # Postgres major version should be >= 17
+        backup_info.set_attribute("summarize_wal", None)
+        if self.postgres.server_version >= 170000:
+            summarize_wal = self.postgres.get_setting("summarize_wal")
+            backup_info.set_attribute("summarize_wal", summarize_wal)
+
+        # Set total size of the PostgreSQL server
+        backup_info.set_attribute("cluster_size", self.postgres.current_size)
+
     @staticmethod
     def _backup_info_from_start_location(backup_info, start_info):
         """
