@@ -201,6 +201,11 @@ def get_snapshot_interface(config):
             config.aws_profile,
             config.aws_region,
             config.aws_await_snapshots_timeout,
+            config.aws_snapshot_lock_mode,
+            config.aws_snapshot_lock_duration,
+            config.aws_snapshot_lock_cool_off_period,
+            config.aws_snapshot_lock_expiration_date,
+            config.tags,
         ]
         return AwsCloudSnapshotInterface(*args)
     else:
@@ -253,6 +258,10 @@ def get_snapshot_interface_from_server_config(server_config):
             server_config.aws_profile,
             server_config.aws_region,
             server_config.aws_await_snapshots_timeout,
+            server_config.aws_snapshot_lock_mode,
+            server_config.aws_snapshot_lock_duration,
+            server_config.aws_snapshot_lock_cool_off_period,
+            server_config.aws_snapshot_lock_expiration_date,
         )
     else:
         raise CloudProviderUnsupported(
@@ -321,10 +330,16 @@ def get_snapshot_interface_from_backup_info(backup_info, config=None):
         # from the backup_info, unless a region is set in the config in which case the
         # config region takes precedence.
         region = None
-        profile = None
-        if config is not None and hasattr(config, "aws_region"):
-            region = config.aws_region
-            profile = config.aws_profile
+        if config is not None:
+            if hasattr(config, "aws_region"):
+                region = config.aws_region
+            try:
+                if getattr(config, "aws_profile"):
+                    profile = config.aws_profile
+            except AttributeError:
+                raise SystemExit(
+                    "Unable to locate credentials. You should configure an AWS profile."
+                )
         if region is None:
             region = backup_info.snapshots_info.region
         return AwsCloudSnapshotInterface(profile, region)
@@ -350,15 +365,11 @@ def snapshots_info_from_dict(snapshots_info):
 
         return GcpSnapshotsInfo.from_dict(snapshots_info)
     elif "provider" in snapshots_info and snapshots_info["provider"] == "azure":
-        from barman.cloud_providers.azure_blob_storage import (
-            AzureSnapshotsInfo,
-        )
+        from barman.cloud_providers.azure_blob_storage import AzureSnapshotsInfo
 
         return AzureSnapshotsInfo.from_dict(snapshots_info)
     elif "provider" in snapshots_info and snapshots_info["provider"] == "aws":
-        from barman.cloud_providers.aws_s3 import (
-            AwsSnapshotsInfo,
-        )
+        from barman.cloud_providers.aws_s3 import AwsSnapshotsInfo
 
         return AwsSnapshotsInfo.from_dict(snapshots_info)
     else:
