@@ -19,16 +19,17 @@
 import logging
 import os
 import re
+from io import SEEK_END, BytesIO, RawIOBase
+
 import requests
-from io import BytesIO, RawIOBase, SEEK_END
 
 from barman.clients.cloud_compression import decompress_to_file
 from barman.cloud import (
+    DEFAULT_DELIMITER,
     CloudInterface,
     CloudProviderError,
     CloudSnapshotInterface,
     DecompressingStreamingIO,
-    DEFAULT_DELIMITER,
     SnapshotMetadata,
     SnapshotsInfo,
     VolumeMetadata,
@@ -43,16 +44,12 @@ except ImportError:
     from urlparse import urlparse
 
 try:
-    from azure.storage.blob import (
-        BlobPrefix,
-        ContainerClient,
-        PartialBatchErrorException,
-    )
     from azure.core.exceptions import (
         HttpResponseError,
         ResourceNotFoundError,
         ServiceRequestError,
     )
+    from azure.storage.blob import ContainerClient, PartialBatchErrorException
 except ImportError:
     raise SystemExit("Missing required python module: azure-storage-blob")
 
@@ -290,26 +287,6 @@ class AzureCloudInterface(CloudInterface):
         # the storage account level in Azure)
         self.container_client.create_container()
 
-    def _walk_blob_tree(self, obj, ignore=None):
-        """
-        Walk a blob tree in a directory manner and return a list of directories
-        and files.
-
-        :param ItemPaged[BlobProperties] obj: Iterable response of BlobProperties
-          obtained from ContainerClient.walk_blobs
-        :param str|None ignore: An entry to be excluded from the returned list,
-          typically the top level prefix
-        :return: List of objects and directories in the tree
-        :rtype: List[str]
-        """
-        if obj.name != ignore:
-            yield obj.name
-        if isinstance(obj, BlobPrefix):
-            # We are a prefix and not a leaf so iterate children
-            for child in obj:
-                for v in self._walk_blob_tree(child):
-                    yield v
-
     def list_bucket(self, prefix="", delimiter=DEFAULT_DELIMITER):
         """
         List bucket content in a directory manner
@@ -322,7 +299,9 @@ class AzureCloudInterface(CloudInterface):
         res = self.container_client.walk_blobs(
             name_starts_with=prefix, delimiter=delimiter
         )
-        return self._walk_blob_tree(res, ignore=prefix)
+
+        for item in res:
+            yield item.name
 
     def download_file(self, key, dest_path, decompress=None):
         """
@@ -506,6 +485,26 @@ class AzureCloudInterface(CloudInterface):
         if errors:
             raise CloudProviderError()
 
+    def get_prefixes(self, prefix):
+        """
+        Return only the common prefixes under the supplied prefix.
+
+        :param str prefix: The object key prefix under which the common prefixes
+            will be found.
+        :rtype: Iterator[str]
+        :return: A list of unique prefixes immediately under the supplied prefix.
+        """
+        raise NotImplementedError()
+
+    def delete_under_prefix(self, prefix):
+        """
+        Delete all objects under the specified prefix.
+
+        :param str prefix: The object key prefix under which all objects should be
+            deleted.
+        """
+        raise NotImplementedError()
+
 
 def import_azure_mgmt_compute():
     """
@@ -750,7 +749,7 @@ class AzureCloudSnapshotInterface(CloudSnapshotInterface):
                 snapshot.identifier, backup_info.snapshots_info.resource_group
             )
 
-    def get_attached_volumes(self, instance_name, disks=None):
+    def get_attached_volumes(self, instance_name, disks=None, fail_on_missing=True):
         """
         Returns metadata for the volumes attached to this instance.
 
@@ -758,8 +757,9 @@ class AzureCloudSnapshotInterface(CloudSnapshotInterface):
         instance and returns a dict of `VolumeMetadata` objects, keyed by disk name.
 
         If the optional disks parameter is supplied then this method returns metadata
-        for the disks in the supplied list only. A SnapshotBackupException is raised if
-        any of the supplied disks are not found to be attached to the instance.
+        for the disks in the supplied list only. If fail_on_missing is set to True then
+        a SnapshotBackupException is raised if any of the supplied disks are not found
+        to be attached to the instance.
 
         If the disks parameter is not supplied then this method returns a
         VolumeMetadata object for every disk attached to this instance.
@@ -768,6 +768,8 @@ class AzureCloudSnapshotInterface(CloudSnapshotInterface):
             are attached.
         :param list[str]|None disks: A list containing the names of disks to be
             backed up.
+        :param bool fail_on_missing: Fail with a SnapshotBackupException if any
+            specified disks are not attached to the instance.
         :rtype: dict[str, VolumeMetadata]
         :return: A dict of VolumeMetadata objects representing each volume
             attached to the instance, keyed by volume identifier.
@@ -784,7 +786,7 @@ class AzureCloudSnapshotInterface(CloudSnapshotInterface):
                 attachment_metadata, disk_metadata
             )
         # Check all requested disks were found and complain if necessary
-        if disks is not None:
+        if disks is not None and fail_on_missing:
             unattached_disks = []
             for disk_name in disks:
                 if disk_name not in attached_volumes:
