@@ -32,7 +32,6 @@ import signal
 import subprocess
 import sys
 import time
-
 from distutils.version import LooseVersion as Version
 
 import barman.utils
@@ -918,6 +917,7 @@ class PgBaseBackup(PostgreSQLClient):
         immediate=False,
         check=True,
         compression=None,
+        parent_backup_manifest_path=None,
         args=None,
         **kwargs
     ):
@@ -937,6 +937,9 @@ class PgBaseBackup(PostgreSQLClient):
           allowed values of the Command obj
         :param barman.compression.PgBaseBackupCompression compression:
           the pg_basebackup compression options used for this backup
+        :param str parent_backup_manifest_path:
+          the path to a backup_manifest file from a previous backup which can
+          be used to perform an incremental backup
         :param List[str] args: additional arguments
         """
         PostgreSQLClient.__init__(
@@ -972,6 +975,10 @@ class PgBaseBackup(PostgreSQLClient):
         if bwlimit is not None and bwlimit > 0:
             self.args.append("--max-rate=%s" % bwlimit)
 
+        # If it has a manifest file path it means it is an incremental backup
+        if parent_backup_manifest_path:
+            self.args.append("--incremental=%s" % parent_backup_manifest_path)
+
         # Immediate checkpoint
         if immediate:
             self.args.append("--checkpoint=fast")
@@ -1142,6 +1149,64 @@ class PgVerifyBackup(PostgreSQLClient):
             self.args += args
 
 
+class PgCombineBackup(PostgreSQLClient):
+    """
+    Wrapper class for the ``pg_combinebackup`` system command
+    """
+
+    COMMAND_ALTERNATIVES = ["pg_combinebackup"]
+
+    def __init__(
+        self,
+        destination,
+        command,
+        tbs_mapping=None,
+        connection=None,
+        version=None,
+        app_name=None,
+        check=True,
+        args=None,
+        **kwargs
+    ):
+        """
+        Constructor
+
+        :param str destination: destination directory path
+        :param str command: the command to use
+        :param None|Dict[str, str] tbs_mapping: used for tablespace
+        :param PostgreSQL connection: an object representing
+          a database connection
+        :param Version version: the command version
+        :param str app_name: the application name to use for the connection
+        :param bool check: check if the return value is in the list of
+          allowed values of the :class:`Command` obj
+        :param None|List[str] args: additional arguments
+        """
+        PostgreSQLClient.__init__(
+            self,
+            connection=connection,
+            command=command,
+            version=version,
+            app_name=app_name,
+            check=check,
+            **kwargs
+        )
+
+        # Set the backup destination
+        self.args = ["--output=%s" % destination]
+
+        # The tablespace mapping option is repeated once for each tablespace
+        if tbs_mapping:
+            for tbs_source, tbs_destination in tbs_mapping.items():
+                self.args.append(
+                    "--tablespace-mapping=%s=%s" % (tbs_source, tbs_destination)
+                )
+
+        # Manage additional args
+        if args:
+            self.args += args
+
+
 class BarmanSubProcess(object):
     """
     Wrapper class for barman sub instances
