@@ -713,10 +713,19 @@ class BackupInfo(FieldListFile):
         # Convert fields which need special types not supported by json
         if data.get("tablespaces") is not None:
             data["tablespaces"] = [list(item) for item in data["tablespaces"]]
+        # Note on the `begin_time_iso`` and `end_time_iso`` fields:
+        # as ctime is not timezone-aware and mostly for human-readable output,
+        # we want to migrate to isoformat for datetime objects representation.
+        # To retain, for now, compatibility with the previous version of the output
+        # we add two new _iso fields to the json document.
         if data.get("begin_time") is not None:
-            data["begin_time"] = data["begin_time"].ctime()
+            begin_time = data["begin_time"]
+            data["begin_time"] = begin_time.ctime()
+            data["begin_time_iso"] = begin_time.isoformat()
         if data.get("end_time") is not None:
-            data["end_time"] = data["end_time"].ctime()
+            end_time = data["end_time"]
+            data["end_time"] = end_time.ctime()
+            data["end_time_iso"] = end_time.isoformat()
         return data
 
     @classmethod
@@ -734,9 +743,15 @@ class BackupInfo(FieldListFile):
             data["tablespaces"] = [
                 Tablespace._make(item) for item in data["tablespaces"]
             ]
-        if data.get("begin_time") is not None:
+        if data.get("begin_time_iso") is not None:
+            data["begin_time"] = load_datetime_tz(data["begin_time_iso"])
+            del data["begin_time_iso"]
+        elif data.get("begin_time") is not None:
             data["begin_time"] = load_datetime_tz(data["begin_time"])
-        if data.get("end_time") is not None:
+        if data.get("end_time_iso") is not None:
+            data["end_time"] = load_datetime_tz(data["end_time_iso"])
+            del data["end_time_iso"]
+        elif data.get("end_time") is not None:
             data["end_time"] = load_datetime_tz(data["end_time"])
         # Instantiate a BackupInfo object using the converted fields
         return cls(server, **data)
@@ -1064,6 +1079,23 @@ class LocalBackupInfo(BackupInfo):
             return True
         return False
 
+    @property
+    def is_orphan(self):
+        """
+        Determine if the backup is an orphan.
+
+        An orphan backup is defined as a backup directory that contains only
+        a non-empty backup.info file. This may indicate an incomplete delete operation.
+
+        :return bool: ``True`` if the backup is an orphan, ``False`` otherwise.
+        """
+        backup_dir = self.get_basebackup_directory()
+        backup_info_path = os.path.join(backup_dir, "backup.info")
+        if os.path.exists(backup_dir) and os.path.exists(backup_info_path):
+            if len(os.listdir(backup_dir)) == 1 and self.status != BackupInfo.EMPTY:
+                return True
+        return False
+
 
 class SyntheticBackupInfo(LocalBackupInfo):
     def __init__(
